@@ -1,4 +1,4 @@
-/* $Id: main.c 28650 2008-11-07 17:28:08Z nick $ */
+/* $Id$ */
 /*
  *  xfce4-settings-manager
  *
@@ -33,10 +33,13 @@
 #include "xfce-settings-manager-dialog.h"
 
 static gboolean opt_version = FALSE;
+static gchar   *opt_dialog = NULL;
 
 static GOptionEntry option_entries[] = {
-    { "version", 'V', 0, G_OPTION_ARG_NONE, &opt_version, 
+    { "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version, 
       N_("Version information"), NULL },
+    { "dialog", 'd', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_STRING, &opt_dialog, 
+      N_("Settings dialog to show"), NULL },
     { NULL }
 };
 
@@ -64,7 +67,7 @@ main(int argc,
 
     if(G_UNLIKELY(opt_version)) {
         g_print("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string());
-        g_print("%s\n", "Copyright (c) 2008");
+        g_print("%s\n", "Copyright (c) 2008-2010");
         g_print("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print(_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print("\n");
@@ -87,9 +90,15 @@ main(int argc,
     g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(gtk_main_quit), NULL);
 
+    if(opt_dialog != NULL) {
+        xfce_settings_manager_dialog_show_dialog(XFCE_SETTINGS_MANAGER_DIALOG(dialog), 
+                                                 opt_dialog);
+    }
+
     gtk_main();
 
-    gtk_widget_destroy(dialog);
+    if(GTK_IS_WIDGET(dialog))
+        gtk_widget_destroy(dialog);
 
     xfconf_shutdown();
 
