@@ -74,6 +74,16 @@
                                | G_PARAM_STATIC_NICK \
                                | G_PARAM_STATIC_BLURB)
 
+#define INCH_MM      25.4
+
+/* Use a fallback DPI of 96 which should be ok-ish on most systems
+ * and is only applied on rare occasions */
+#define FALLBACK_DPI 96
+
+/* Use the same min/max DPI as in the appearance settings dialog */
+#define MIN_DPI      48
+#define MAX_DPI      1000
+
 G_DEFINE_TYPE(XSettingsRegistry, xsettings_registry, G_TYPE_OBJECT);
 
 enum
@@ -107,6 +117,7 @@ enum
     XSETTING_ENTRY_GTK_MENUBARACCEL,
     XSETTING_ENTRY_GTK_CURSORTHEMENAME,
     XSETTING_ENTRY_GTK_CURSORTHEMESIZE,
+    XSETTING_ENTRY_GTK_IMMODULE,
 } XSettingType;
 
 static XSettingsRegistryEntry properties[] = {
@@ -141,6 +152,7 @@ static XSettingsRegistryEntry properties[] = {
 { "Gtk/MenuBarAccel", {G_TYPE_STRING, }},
 { "Gtk/CursorThemeName", {G_TYPE_STRING, }},
 { "Gtk/CursorThemeSize", {G_TYPE_INT, }},
+{ "Gtk/IMModule", {G_TYPE_STRING, }},
 
 { NULL, {0, }},
 };
@@ -167,6 +179,8 @@ static void
 cb_xsettings_registry_channel_property_changed(XfconfChannel *channel, const gchar *property_name, const GValue *value, XSettingsRegistry *registry); 
 static Bool
 timestamp_predicate (Display *display, XEvent  *xevent, XPointer arg);
+static int
+compute_xsettings_dpi (XSettingsRegistry *registry);
 
 gboolean
 xsettings_registry_process_event (XSettingsRegistry *registry, XEvent *xevent)
@@ -257,7 +271,7 @@ xsettings_registry_init(XSettingsRegistry *registry)
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_FONTNAME].value, "Sans 10");
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_ICONSIZES].value, NULL);
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_KEYTHEMENAME].value, NULL);
-    g_value_set_string (&properties[XSETTING_ENTRY_GTK_TOOLBARSTYLE].value, "Icons");
+    g_value_set_string (&properties[XSETTING_ENTRY_GTK_TOOLBARSTYLE].value, "icons");
     g_value_set_int (&properties[XSETTING_ENTRY_GTK_TOOLBARICONSIZE].value, 3);
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_IMPREEDITSTYLE].value, "");
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_IMSTATUSSTYLE].value, "");
@@ -267,6 +281,7 @@ xsettings_registry_init(XSettingsRegistry *registry)
 
     g_value_set_string (&properties[XSETTING_ENTRY_GTK_CURSORTHEMENAME].value, NULL);
     g_value_set_int (&properties[XSETTING_ENTRY_GTK_CURSORTHEMESIZE].value, 0);
+    g_value_set_string (&properties[XSETTING_ENTRY_GTK_IMMODULE].value, NULL);
 }
 
 static void
@@ -283,6 +298,7 @@ cb_xsettings_registry_channel_property_changed(XfconfChannel *channel, const gch
             break;
         }
     }
+
     xsettings_registry_notify(registry);
     if (!strncmp(name, "/Xft", 4) || !strncmp(name, "/Gtk/CursorTheme", 16))
         xsettings_registry_store_xrdb(registry);
@@ -316,7 +332,7 @@ xsettings_registry_store_xrdb(XSettingsRegistry *registry)
             g_string_append_printf (string, "Xft.hintstyle: %s\n", g_value_get_string (&properties[XSETTING_ENTRY_XFT_HINTSTYLE].value));
         else
             string = g_string_append (string, "Xft.hintstyle: hintnone\n");
-            
+        
         if (g_value_get_int (&properties[XSETTING_ENTRY_XFT_DPI].value) > 0)
             g_string_append_printf (string, "Xft.dpi: %d\n", g_value_get_int (&properties[XSETTING_ENTRY_XFT_DPI].value));
 
@@ -399,16 +415,43 @@ xsettings_registry_store_xrdb(XSettingsRegistry *registry)
     g_error_free (error);
 }
 
+static int
+compute_xsettings_dpi (XSettingsRegistry *registry)
+{
+    Screen *xscreen;
+    int width_mm, height_mm;
+    int width, height;
+    int dpi;
+    
+    xscreen = ScreenOfDisplay (registry->priv->display,
+                                registry->priv->screen);
+    width_mm = WidthMMOfScreen (xscreen);
+    height_mm = HeightMMOfScreen (xscreen);
+    dpi = FALLBACK_DPI;
+    
+    if (width_mm > 0 && height_mm > 0)
+    {
+        width = WidthOfScreen (xscreen);
+        height = HeightOfScreen (xscreen);
+        dpi = MIN (INCH_MM * width  / width_mm,
+                   INCH_MM * height / height_mm);
+    }
+
+    return dpi;
+}
+
 void
 xsettings_registry_notify(XSettingsRegistry *registry)
 {
     guchar *buffer, *pos;
     gint buf_len;
-
     gint prop_count = 0;
-    registry->priv->last_change_serial = registry->priv->serial;
-
     XSettingsRegistryEntry *entry = properties;
+    const gchar *value, *val;
+    gint name_len, value_len, str_length;
+    gint dpi;
+    
+    registry->priv->last_change_serial = registry->priv->serial;
 
     buf_len = 12;
 
@@ -426,7 +469,7 @@ xsettings_registry_notify(XSettingsRegistry *registry)
             case G_TYPE_STRING:
                 {
                     buf_len += 4;
-                    const gchar *value = g_value_get_string(&entry->value);
+                    value = g_value_get_string(&entry->value);
                     if(value)
                     {
                         buf_len += XSETTINGS_PAD(strlen(value), 4);
@@ -460,9 +503,6 @@ xsettings_registry_notify(XSettingsRegistry *registry)
     entry = properties;
     for(; entry->name != NULL; ++entry)
     {
-        gint name_len, value_len = 0, str_length;
-
-
         name_len = XSETTINGS_PAD(strlen(entry->name), 4);
         value_len = 0;
 
@@ -475,7 +515,7 @@ xsettings_registry_notify(XSettingsRegistry *registry)
             case G_TYPE_STRING:
                 *pos++ = 1; // String 
                 {
-                    const gchar *value = g_value_get_string(&entry->value);
+                    value = g_value_get_string(&entry->value);
                     if(value)
                     {
                         value_len = XSETTINGS_PAD(strlen(value), 4);
@@ -512,7 +552,7 @@ xsettings_registry_notify(XSettingsRegistry *registry)
         {
             case G_TYPE_STRING:
                 {
-                    const gchar *val = g_value_get_string(&entry->value);
+                    val = g_value_get_string(&entry->value);
 
                     if (val)
                     {
@@ -541,8 +581,23 @@ xsettings_registry_notify(XSettingsRegistry *registry)
                  * font sizes with -1 DPI and a forced setting equal to the X
                  * server's calculated DPI don't match.  Need to look into
                  * this a bit more. */
-                if (strcmp (entry->name, "Xft/DPI") == 0 && g_value_get_int(&entry->value) != -1)
-                    *(CARD32 *)pos = g_value_get_int(&entry->value) * 1024;
+                if (strcmp (entry->name, "Xft/DPI") == 0)
+                {
+                    dpi = g_value_get_int (&entry->value);
+
+                    if (dpi < 0)
+                    {
+                        /* Compute the DPI based on X */
+                        dpi = compute_xsettings_dpi (registry);
+                    }
+
+                    /* Make sure to use the fallback DPI if the user-defined or computed 
+                     * value is out of range */
+                    dpi = dpi < MIN_DPI ? FALLBACK_DPI : (dpi > MAX_DPI ? FALLBACK_DPI : dpi);
+
+                    /* Apply the new value */
+                    *(CARD32 *)pos = 1024 * dpi;
+                }
                 else
                     *(CARD32 *)pos = g_value_get_int(&entry->value);
                 pos += 4;
